#include "Ballet128256.h"  

#include "Ballet128256_coreE.h"
#include "Ballet128256_coreD.h"

#define BytesperEncS 16
static void EncBallet128_256B_1W_Round(unsigned char *out,unsigned char *pt,  unsigned char *rk, int round);
static void DecBallet128_256B_1W_Round(unsigned char *out,unsigned char *pt,  unsigned char *rk, int round);

static void BalletGenRK_128_256(u8i * Rk, u8i * K,int dirt);   
static void Ballet128256EncDataS_Round(u8i *out, u8i *in, u8i *K, int len,int round);


/*
Ballet 128-256  interface 
*/

int Crypt_Enc_Block(unsigned char *input,int in_len, unsigned char  *output, int *out_len, unsigned char *key , int keylen)
{
	u32i rk[RoundBallet128256 * 4];
	int i;
	if(in_len<1)
		return -1;
	if(keylen!=32)
		return -2;  
	BalletGenRK_128_256_ENC((u8i *)rk, key);
	for (i = 0; i < in_len; i += 16)
		Ballet128256EncDataS(output + i, input + i, (u8i *)rk);
	 
	*out_len= in_len;
	return 0;
}

int Crypt_Dec_Block(unsigned char *input,int in_len, unsigned char *output,int *out_len, unsigned char *key, int keylen)
{
	__m256i rkm[96];
	u8i TText[256];
	int leftbytes;
	if (in_len < 1)
		return -1;
	if (keylen != 32)
		return -2;
	BalletGenRK_128_256M_DEC(rkm, key);
	leftbytes = in_len;
	while (leftbytes >= 256)
	{
		Ballet128256DecDataM(output, input, (u8i *)rkm);
		output += 256;
		input +=256;
		leftbytes -= 256;
	}
	if (leftbytes)
	{
		memcpy(TText, input, leftbytes);
		Ballet128256DecDataM(TText, TText, (u8i *)rkm);
		memcpy(output, TText, leftbytes);
	}   

	*out_len= in_len;
	return 0;
}

//IV ȫ 0

int Crypt_Enc_Block_CBC(unsigned char *input, int in_len, unsigned char  *output, int *out_len, unsigned char *key, int keylen)
{
	u32i _C_ALIGN32(rk[RoundBallet128256 * 2]);
	u64i _C_ALIGN16(IV[2]);
	int i ;
	if (in_len < 1)
		return -1;
	if (in_len & 0x0f)
		return -2;
	if (keylen != 32)
		return -3;
	BalletGenRK_128_256_ENC((u8i *)rk, key);
	Ballet128256EncDataS(output, input, (u8i *)rk);
	for (i = 16; i < in_len; i += 16)
	{
		IV[0] = (*(u64i *)(input + i)) ^ (*(u64i *)(output + i - 16));
		IV[1] = (*(u64i *)(input + i + 8)) ^ (*(u64i *)(output + i - 8));
		Ballet128256EncDataS(output + i, (u8i *)IV, (u8i *)rk);
	}
	*out_len = in_len;
	return 0;
}

int Crypt_Dec_Block_CBC(unsigned char *input, int in_len, unsigned char  *output, int *out_len, unsigned char *key, int keylen)
{
	u8i _C_ALIGN16(TText[256]);
	__m256i rkm[RoundBallet128256 * 2];
	__m128i mIV = _mm_set1_epi64x(0);
	int  leftbytes;
	if (in_len < 1)
		return -1;
	if (in_len & 0x0f)
		return -2;
	if (keylen != 32)
		return -3;
	BalletGenRK_128_256M_DEC( rkm, key);
	leftbytes = in_len;
	while (leftbytes >= 256)
	{
		Ballet128256DecDataM_CBC(output, input, (u8i *)rkm, &mIV);
		output += 256;
		input += 256;
		leftbytes -= 256;
	}
	if (leftbytes)
	{
		memcpy(TText, input, leftbytes);
		Ballet128256DecDataM_CBC(TText, TText, (u8i *)rkm, &mIV);
		memcpy(output, TText, leftbytes);
	}
	*out_len = in_len;
	return 0;
}

//ֻּӽ
//ӽԿ洢ռ䲻ͬ 
void Key_Schedule(unsigned char *Seedkey, int KeyLen, unsigned   char Direction, unsigned char *Subkey)
{
	if (Direction == Dirt_ENC)
		BalletGenRK_128_256_ENC(Subkey, Seedkey);
	else
		BalletGenRK_128_256M_DEC((__m256i *)Subkey, Seedkey);
}


int Crypt_Enc_Block_Round(unsigned char *input,int in_len,  unsigned char *output, int *out_len, unsigned char *key, int keylen, int CryptRound)
{
	if(in_len<1)
		return -1;
	if(keylen!=32)
		return -2; 
	if((CryptRound> RoundBallet128256) || (CryptRound<0))
		return -3;
	Ballet128256EncDataS_Round(output, input, key, in_len,CryptRound);
	*out_len= in_len;
	return 0;
}

/*
end of ballet 128-256 interface
*/
static void BalletGenRK_128_256(u8i * Rk, u8i * K,int dirt)
{
	u64i i;
	u64i  k0,k1,t0,t1;
	u64i *p=(u64i *)Rk;
	k0= *((u64i *)K);
	k1= *((u64i *)(K+8));
	t0= *((u64i *)(K+16));
	t1= *((u64i *)(K+24)); 
	k0 =SWAP64(k0);
	k1 =SWAP64(k1);
	t0 =SWAP64(t0);
	t1 =SWAP64(t1);  
	if(dirt==Dirt_ENC)
	{
		for(  i=0;i<RoundBallet128256;i++)  
		{
			if(i%2==0)
			{
				*p= k0;  
				t0^=  ROTL64(t1,7)^ROTL64(t1,17); 
				k0^=  ROTL64(k1,3)^ROTL64(k1,5);  
				k0^=  t0^i;
			}
			else
			{
				*(p)= k1; 
				t1^=  ROTL64(t0,7)^ROTL64(t0,17); 
				k1^=  ROTL64(k0,3)^ROTL64(k0,5);  
				k1^=  t1^(i);	 
			}
			p++;
		}
	}
	else
	{
		p+=RoundBallet128256-1;
		for(  i=0;i<RoundBallet128256;i++)  
		{
			if(i%2==0)
			{
				*p= k0; 
				t0^=  ROTL64(t1,7)^ROTL64(t1,17); 
				k0^=  ROTL64(k1,3)^ROTL64(k1,5);  
				k0^=  t0^i;
			}
			else
			{
				*(p)= k1;  
				t1^=  ROTL64(t0,7)^ROTL64(t0,17); 
				k1^=  ROTL64(k0,3)^ROTL64(k0,5);  
				k1^=  t1^(i);	 
			}
			p--;
		}
	}

}


static void EncBallet128_256B_1W_Round(unsigned char *out,unsigned char *pt,  unsigned char *rk, int round)
{
	int i;
	u32i t0; 
	u32i *trk = (u32i *)rk; 
	u32i a = *((u32i *)pt);
	u32i b = *((u32i *)(pt+4));
	u32i c = *((u32i *)(pt+8));	
	u32i d = *((u32i *)(pt+12));	

	a =SWAP32(a);b =SWAP32(b);
	c =SWAP32(c); d =SWAP32(d);  
	for(  i=0;i<round;i++)
	{    
		if(i%2==0)
		{
			t0= b^c;   	
			b^=  (*(trk+1) );
			c^= (*trk);  
			d= ROTL32(d,15);  
			a=ROTL32(a,6); 
			a+=ROTL32(t0,9); 
			d+=ROTL32(t0,14);
		}
		else
		{
			t0= a^d;  
			b=ROTL32(b,6);
			b+=ROTL32(t0,9);  
			d^= (*(trk));
			a^= (*(trk+1));   
			c= ROTL32(c,15);  
			c+=ROTL32(t0,14); 	
		}
		trk+=2;

	}
	a =SWAP32(a);b =SWAP32(b);
	c =SWAP32(c); d =SWAP32(d);
	if(round%2)
	{
		*((u32i *)out)=a;
		*((u32i *)(out+4))=b;
		*((u32i *)(out+8))=c;	
		*((u32i *)(out+12))=d;
	}
	else
	{ 
		*((u32i *)out)=b;
		*((u32i *)(out+4))=a;
		*((u32i *)(out+8))=d;	
		*((u32i *)(out+12))=c;

	}			
} 

static void DecBallet128_256B_1W_Round(unsigned char *out,unsigned char *pt,  unsigned char *rk,int round)
{
	int i;
	u32i t0; 
	u32i *trk = (u32i *)rk; 
	u32i a = *((u32i *)pt);
	u32i b = *((u32i *)(pt+4));
	u32i c = *((u32i *)(pt+8));	
	u32i d = *((u32i *)(pt+12)); 
	a =SWAP32(a);b =SWAP32(b);
	c =SWAP32(c); d =SWAP32(d);  
	for(  i=0;i<round;i++)
	{         	
		if(i%2==0)
		{
			b^=  (*(trk+1) );
			c^= (*trk);  
			t0=b^c; 
			a-=ROTL32(t0,9);
			d-=ROTL32(t0,14);
			a= ROTL32(a,32-6);
			d= ROTL32(d,32-15); 
		}
		else
		{
			a^=  (*(trk+1));
			d^= (*(trk));  
			t0= a^d;
			b-=ROTL32(t0,9);
			c-=ROTL32(t0,14);	
			b= ROTL32(b,32-6);
			c= ROTL32(c,32-15); 		
		}
		trk+=2;

	}
	a =SWAP32(a);b =SWAP32(b);
	c =SWAP32(c); d =SWAP32(d);
	if(round%2)
	{

		*((u32i *)out)=a;
		*((u32i *)(out+4))=b;
		*((u32i *)(out+8))=c;	
		*((u32i *)(out+12))=d;	
	}
	else
	{

		*((u32i *)out)=b;
		*((u32i *)(out+4))=a;
		*((u32i *)(out+8))=d;	
		*((u32i *)(out+12))=c;		
	}			
}



 


static void Ballet128256EncDataS_Round(u8i *out, u8i *in, u8i *K, int len,int round)
{ 

	u32i rk[RoundBallet128256*2];
	int i ;
	BalletGenRK_128_256((u8i *)rk,K,Dirt_ENC);
	for(  i=0;i<len;i+=BytesperEncS)  
		EncBallet128_256B_1W_Round(out+i, in+i, (u8i *)rk,round); 
#ifdef OutputInnerPars
	printf("round %02d : ", round-1);
	for(  j=0;j<16;j++)
	{
		printf("%02X", out[j]);
				   if((j+1)%4==0)
			   printf(" ");
	}
	printf(" rk%02d: ",round-1);
	printf("%08X ", rk[round*2-1]); 
	printf("%08X ", rk[round*2-2]); 
	printf("\n"); 
#endif

}